/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.kernel.plugin;

import com.google.common.base.Function;
import com.google.common.collect.Iterables;
import com.google.inject.AbstractModule;
import com.google.inject.Injector;
import com.google.inject.Module;
import com.sos.scheduler.engine.kernel.command.CommandDispatcher;
import com.sos.scheduler.engine.kernel.log.PrefixLog;
import com.sos.scheduler.engine.kernel.scheduler.SchedulerException;
import org.w3c.dom.Element;

import javax.annotation.Nullable;

import static com.google.common.base.Preconditions.checkState;
import static com.google.common.base.Throwables.propagate;
import static com.sos.scheduler.engine.kernel.util.Util.optional;
import static com.sos.scheduler.engine.kernel.util.XmlUtils.xmlQuoted;

/** Die Engine spricht das Plugin über diesen Adapter an. */
class PluginAdapter {
    private final PluginConfiguration configuration;
    private final PrefixLog log;
    @Nullable private Plugin plugin = null;
    private boolean isActive = false;

    PluginAdapter(PluginConfiguration c, PrefixLog log) {
        this.configuration = c;
        this.log = log;
    }

    final void tryActivate(Injector injector) {
        try {
            activate(injector);
        } catch (Exception x) {
            logThrowable(x);
        }
    }

    final void activate(Injector injector) {
        checkState(!isActive, this +" is already active");
        if (plugin == null)
            plugin = newPlugin(injector);
        plugin.activate();
        isActive = true;
    }

    final String getXmlState() {
        try {
            return getPlugin().getXmlState();
        } catch (Exception x) {
            return "<ERROR text="+ xmlQuoted(x.toString()) +"/>";
        }
    }

    final void tryClose() {
        try {
            if (plugin != null)
                plugin.close();
            isActive = false;
        } catch (Exception x) {
            logThrowable(x);
        }
    }

    private Plugin newPlugin(Injector injector) {
        return newPluginByDI(injector, pluginClassForName(configuration.getClassName()), configuration.getConfigElementOrNull());
    }

    private static Class<? extends Plugin> pluginClassForName(String name) {
        try {
            Class<?> c = Class.forName(name);
            if (!Plugin.class.isAssignableFrom(c))
                throw new SchedulerException("Plugin does not implement " + Plugin.class.getName());

            @SuppressWarnings("unchecked")
            Class<? extends Plugin> result = (Class<? extends Plugin>)c;
            return result;
        } catch (ClassNotFoundException x) { throw propagate(x); }
    }

    private static Plugin newPluginByDI(Injector injector, Class<? extends Plugin> c, @Nullable Element elementOrNull) {
        Injector myInjector = injector.createChildInjector(Iterables.transform(optional(elementOrNull), elementToModule()));
        return myInjector.getInstance(c);
    }

    private static Function<Element,Module> elementToModule() {
        // Funktion, damit Abhängigkeit zu DI-Jars erst zur Laufzeit, bei Bedarf, verlangt werden.
        return new Function<Element,Module>() {
            @Override public Module apply(final Element e) {
                return new AbstractModule() {
                    @Override protected void configure() {
                        bind(Element.class).toInstance(e);
                    }
                };
            }
        };
    }

    final String getPluginClassName() {
        return configuration.getClassName();
    }

    final Plugin getPlugin() {
        if (plugin == null)  throw new IllegalStateException(this +" is not loaded");
        return plugin;
    }

    final CommandDispatcher getCommandDispatcher() {
        Plugin p = getPlugin();
        if (!(p instanceof CommandPlugin))
            throw new SchedulerException("Plugin is not a " + CommandPlugin.class.getSimpleName());
        return new CommandDispatcher(((CommandPlugin)p).getCommandHandlers());
    }

    public final PluginConfiguration getConfiguration() {
        return configuration;
    }

    private void logThrowable(Throwable t) {
        PluginSubsystem.logError(log, toString(), t);
    }

    @Override public String toString() {
        return "Plugin "+ configuration.getClassName();
    }
}
